﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace TestGenerateTilingImages
{
    public partial class Form1 : Form
    {
        string ecwFileName = @"../../DataFolder/World.ecw";
        string cacheDirectory = @"../../CacheFolder";
        
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Collection<ZoomLevel> zoomLevels = GetAllZoomLevels();
            int tileWidth = 512;
            int tileHeight = 512;
            
            EcwRasterLayer ecwImageLayer = new EcwRasterLayer(ecwFileName);
            ecwImageLayer.UpperThreshold = double.MaxValue;
            ecwImageLayer.LowerThreshold = 0;

            MapEngine mapEngine = new MapEngine();
            mapEngine.StaticLayers.Add(ecwImageLayer);
            GeographyUnit mapUnit = GeographyUnit.DecimalDegree;

            ecwImageLayer.Open();
            RectangleShape cacheExtent = ecwImageLayer.GetBoundingBox();
            ecwImageLayer.Close();

            //for (int i = 0; i < zoomLevels.Count; i++)
            for (int i = 0; i < 5; i++)
            {
                MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(zoomLevels[i].Scale, tileWidth, tileHeight, mapUnit);
                BitmapTileCache tileCache = new FileBitmapTileCache(cacheDirectory, string.Empty, TileImageFormat.Png, tileMatrix);
                RowColumnRange rowColumnRange = tileCache.TileMatrix.GetIntersectingRowColumnRange(cacheExtent);

                for (long rowIndex = rowColumnRange.MinRowIndex; rowIndex <= rowColumnRange.MaxRowIndex; rowIndex++)
                {
                    for (long columnIndex = rowColumnRange.MinColumnIndex; columnIndex <= rowColumnRange.MaxColumnIndex; columnIndex++)
                    {
                        Bitmap bitmap = new Bitmap(tileCache.TileMatrix.TileWidth, tileCache.TileMatrix.TileHeight);
                        mapEngine.CurrentExtent = tileCache.TileMatrix.GetCell(rowIndex, columnIndex).BoundingBox;
                        mapEngine.ShowLogo = false;
                        mapEngine.OpenAllLayers();
                        mapEngine.DrawStaticLayers(bitmap, mapUnit);
                        mapEngine.DrawDynamicLayers(bitmap, mapUnit);
                        mapEngine.CloseAllLayers();

                        tileCache.SaveTiles(bitmap, mapEngine.CurrentExtent);
                        bitmap.Dispose();
                    }
                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-118.098, 84.3, 118.098, -84.3);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            EcwRasterLayer ecwImageLayer = new EcwRasterLayer(ecwFileName);
            ecwImageLayer.UpperThreshold = double.MaxValue;
            ecwImageLayer.LowerThreshold = 0;

            LayerOverlay imageOverlay = new LayerOverlay();
            FileBitmapTileCache fileBitmapTileCache = new FileBitmapTileCache(cacheDirectory, string.Empty);
            fileBitmapTileCache.TileMatrix.TileWidth = 512;
            fileBitmapTileCache.TileMatrix.TileHeight = 512;

            imageOverlay.TileCache = fileBitmapTileCache;
            imageOverlay.Layers.Add("EcwImageLayer", ecwImageLayer);
            winformsMap1.Overlays.Add(imageOverlay);

            winformsMap1.Refresh();
        }   

        private Collection<ZoomLevel> GetAllZoomLevels()
        {
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            Collection<ZoomLevel> allZoomLevels = new Collection<ZoomLevel>();
            allZoomLevels.Add(zoomLevelSet.ZoomLevel01);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel02);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel03);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel04);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel05);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel06);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel07);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel08);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel09);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel10);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel11);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel12);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel13);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel14);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel15);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel16);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel17);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel18);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel19);
            allZoomLevels.Add(zoomLevelSet.ZoomLevel20);
            return allZoomLevels;
        }
    }
}